package com.fossgalaxy.games.tbs.actions;

import com.fossgalaxy.games.tbs.GameState;
import com.fossgalaxy.games.tbs.entity.Entity;
import com.fossgalaxy.games.tbs.order.MultiOrder;
import com.fossgalaxy.games.tbs.order.Order;
import com.fossgalaxy.games.tbs.ui.GameAction;
import com.fossgalaxy.object.annotations.ObjectDef;
import org.codetome.hexameter.core.api.CubeCoordinate;

import java.awt.*;
import java.util.Arrays;

public class MultiAction implements GameAction {

    private final GameAction[] actions;

    @ObjectDef("MultiAction")
    public MultiAction(GameAction[] actions) {
        this.actions = actions;
    }

    @Override
    public void renderHints(Graphics2D g, GameState s, Entity actor) {
        for(GameAction action : actions){
            action.renderHints(g, s, actor);
        }
    }

    @Override
    public Order generateOrder(CubeCoordinate co, GameState s) {
        Order[] orders = new Order[actions.length];
        for(int i = 0; i < actions.length; i++){
            orders[i] = actions[i].generateOrder(co, s);
        }
        return new MultiOrder(orders);
    }

    @Override
    public boolean isPossible(Entity entity, GameState state) {
        for(GameAction action : actions){
            if(!action.isPossible(entity, state)) return false;
        }
        return true;
    }

    @Override
    public Color getHintColour() {
        return actions[0].getHintColour();
    }

    @Override
    public Color getBorderColour() {
        return actions[0].getHintColour();
    }

    @Override
    public String toString(){
        return Arrays.toString(actions);
    }
}
