package com.fossgalaxy.games.tbs.rules;

import com.fossgalaxy.games.tbs.GameState;
import com.fossgalaxy.games.tbs.entity.Entity;
import com.fossgalaxy.games.tbs.parameters.EntityType;
import com.fossgalaxy.object.annotations.ObjectDef;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

// Must have at least threshold of keepThisAlive
public class KeepAlive implements Rule {

    private final String keepThisAlive;
    private final int threshold;


    @ObjectDef("KeepAlive")
    public KeepAlive(String keepThisAlive, int threshold) {
        this.keepThisAlive = keepThisAlive;
        this.threshold = threshold;
    }

    @Override
    public List<Integer> getLosers(GameState state) {
        EntityType type = state.getSettings().getEntityType(keepThisAlive);
        Map<Integer, Long> counts = state.getEntities().stream()
                .filter(x -> x.getType().isInstance(type))
                .collect(Collectors.groupingBy(Entity::getOwner, Collectors.counting()));

        System.out.println(counts);
        List<Integer> results = null;

        for(int player = 0; player < state.getNumberOfPlayers(); player++){
            long count = counts.getOrDefault(player, 0L);
            if(count < threshold) {
                if(results == null) results = new ArrayList<>();
                results.add(player);
            }
        }
//        for (Map.Entry<Integer, Long> entry : counts.entrySet()) {
//            if (entry.getValue() < threshold) {
//                if (results == null) results = new ArrayList<>();
//                results.add(entry.getKey());
//            }
//        }
        return results == null ? Collections.emptyList() : results;
    }
}
